@@ -13,19 +13,19 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-import pygtk
-pygtk.require('2.0')
-import gtk
-import gobject
 
 import xml.dom.minidom
 from xml.sax.saxutils import unescape
 
+import gtk
+import gobject
+
 from gazpacho.loader import tags
 from gazpacho.loader import compat
 from gazpacho.gaction import GAction
 from gazpacho.l10n import _
 from gazpacho import util
+from gazpacho.choice import enum_from_string, flags_from_string
 
 def dump_widget(widget, indent=0):
     print ' '*indent, widget
@@ -51,6 +51,8 @@ class WidgetTree(object):
         self.loaders['GtkAction'] = self._build_action
         self.loaders['GtkMenuBar'] = self._build_menu_bar_or_toolbar
         self.loaders['GtkToolbar'] = self._build_menu_bar_or_toolbar
+        self.loaders['GtkFrame'] = self._build_frame
+        self.loaders['has-default'] = self._read_has_default
 
         self._reset()
         self._placeholder = placeholder
@@ -69,6 +71,11 @@ class WidgetTree(object):
         self._uimanager = None
 
         self._deprecated_widgets = []
+
+        # here we store a list of functions we need to call after the
+        # loading of a toplevel is done. For example setting some properties
+        # that need the object to be added to the window ('has-default')
+        self._postload_tasks = []
         
     def load_from_file(self, path):
         self._reset()
@@ -85,13 +92,32 @@ class WidgetTree(object):
         self._reset()
 
         doc = xml.dom.minidom.parseString(stream)
-
         self.load_from_xml_document_instance(doc)
 
     def load_from_xml_document_instance(self, document):
         self._read_xml_tree(document.documentElement)
         self._set_up_accel_groups()
-        
+
+    def load_widget_from_stream(self, stream):
+        pass
+    
+    def load_widget_from_document_instance(self, document):
+        if xml_root_node.tagName != tags.XML_TAG_PROJECT:
+            if child_node.tagName == tags.XML_TAG_REQUIRES:
+                self._add_requirement(child_node)
+
+            elif child_node.tagName == tags.XML_TAG_OBJECT:
+                self._build_object(child_node)
+            
+            elif child_node.tagName == tags.XML_TAG_WIDGET:
+                widget = self._build_object(child_node)
+                if widget is not None:
+                    # make window management easier by making created windows
+                    # transient for the editor window
+                    if self._app is not None \
+                           and isinstance(widget, gtk.Window):
+                        widget.set_transient_for(self._app.window)
+    
     def __getattr__(self, name):
         if name in self._widgets.keys():
             return self._widgets[name]
@@ -105,20 +131,24 @@ class WidgetTree(object):
         # read the toplevel widgets (each toplevel read its own children)
         for child_node in util.xml_filter_nodes(xml_root_node.childNodes,
                                                 xml_root_node.ELEMENT_NODE):
+            self._postload_tasks = []
             if child_node.tagName == tags.XML_TAG_REQUIRES:
                 self._add_requirement(child_node)
 
             elif child_node.tagName == tags.XML_TAG_OBJECT:
-                self._build_object(child_node)
+                obj = self._build_object(child_node)
+                if obj is not None:
+                    self._run_postload_tasks(obj)
             
             elif child_node.tagName == tags.XML_TAG_WIDGET:
                 widget = self._build_object(child_node)
                 if widget is not None:
+                    self._run_postload_tasks(widget)
                     # make window management easier by making created windows
                     # transient for the editor window
-                    if self._app is not None:
+                    if self._app is not None \
+                           and isinstance(widget, gtk.Window):
                         widget.set_transient_for(self._app.window)
-                    widget.show_all()
 
         # check if we have any deprecated widget
         if self._deprecated_widgets:
@@ -127,7 +157,13 @@ class WidgetTree(object):
                                                         self._deprecated_widgets)
                 dialog.run()
                 dialog.destroy()
-                
+
+    def _run_postload_tasks(self, toplevel):
+        for task in self._postload_tasks:
+            f = task[0]
+            args = task[1:]
+            f(*args)
+            
     def _add_requirement(self, xml_node):
         assert xml_node.tagName == tags.XML_TAG_REQUIRES
 
@@ -200,7 +236,7 @@ class WidgetTree(object):
             # build the children
             elif child_node.tagName == tags.XML_TAG_CHILD:
                 child = self._build_child_object(obj, child_node)
-                if child is not None:
+                if child is not None and child not in obj.get_children():
                     obj.add(child)
                     self._read_packing_properties(obj, child, child_node)
     
@@ -219,9 +255,19 @@ class WidgetTree(object):
             elif child_node.tagName == tags.XML_TAG_WIDGET:
                 internal_child = xml_node.getAttribute(tags.XML_TAG_INTERNAL_CHILD)
                 if internal_child:
-                    obj = getattr(parent, internal_child)
+                    # search for the ancestor that we are internal child of
+                    # note that this is not always the direct parent
+                    ancestor = parent
+                    while getattr(ancestor, internal_child, None) is None:
+                        ancestor = ancestor.get_parent()
+                    
+                    obj = getattr(ancestor, internal_child)
                     self._fill_object_from_node(obj, child_node)
                     obj.set_data('internal_child', internal_child)
+                    # usually the name is set in _build_object but as this
+                    # is an internal widget we do it manually
+                    name = child_node.getAttribute(tags.XML_TAG_ID)
+                    obj.set_name(name)
                 else:
                     obj = self._build_object(child_node)
                 break
@@ -237,9 +283,8 @@ class WidgetTree(object):
         for child_node in util.xml_filter_nodes(xml_node.childNodes,
                                                 xml_node.ELEMENT_NODE):
             if child_node.tagName == tags.XML_TAG_PACKING:
-                for prop_node in child_node.childNodes:
-                    if not prop_node.nodeType == prop_node.ELEMENT_NODE:
-                        continue
+                for prop_node in util.xml_filter_nodes(child_node.childNodes,
+                                                       child_node.ELEMENT_NODE):
                     if prop_node.tagName == tags.XML_TAG_PROPERTY:
                         self._read_child_property(child, prop_node, parent)
     
@@ -328,14 +373,57 @@ class WidgetTree(object):
             
         obj.set_property('name', id)
         return obj
-    
+
+    def _build_frame(self, xml_node):
+        id = xml_node.getAttribute(tags.XML_TAG_ID)        
+        obj = gtk.Frame()
+        obj.set_property('name', id)
+        for child_node in util.xml_filter_nodes(xml_node.childNodes,
+                                                xml_node.ELEMENT_NODE):
+            # read the properties
+            if child_node.tagName == tags.XML_TAG_PROPERTY:
+                self._read_property(obj, child_node)
+
+            # read the signals
+            elif child_node.tagName == tags.XML_TAG_SIGNAL:
+                self._read_signal(obj, child_node)
+                
+            # build the children
+            elif child_node.tagName == tags.XML_TAG_CHILD:
+                child = self._build_child_object(obj, child_node)
+                if child is not None:
+                    type_node = util.xml_find_node(child_node,
+                                                    tags.XML_TAG_PROPERTY,
+                                                    name='type')
+                    if type_node is not None:
+                        value = util.xml_read_raw_property(type_node)[1]
+                        if value == 'label_item':
+                            obj.set_label_widget(child)
+                            continue
+
+                    obj.add(child)
+                    self._read_packing_properties(obj, child, child_node)    
+
+        return obj
+
+    def _set_has_default(self, obj, value):
+        obj.set_property('has-default', value)
+        
+    def _read_has_default(self, obj, value):
+        """Can't set the default until the object is added to the toplevel
+        window"""
+        self._postload_tasks.append((self._set_has_default, obj, value))
+        
     def _read_property(self, obj, xml_node):
         name, value = util.xml_read_raw_property(xml_node)
         for pspec in gobject.list_properties(obj):
             if pspec.name == name:
-                value = self._value_from_string(value, pspec.value_type)
+                value = self._value_from_string(value, pspec)
                 if value is not None:
-                    obj.set_property(name, value)
+                    if name in self.loaders.keys():
+                        self.loaders[name](obj, value)
+                    else:
+                        obj.set_property(name, value)
                     break
 
         # Read any i18n metadata and set as object data so the
@@ -343,7 +431,7 @@ class WidgetTree(object):
         # created
         value = xml_node.getAttribute(tags.XML_TAG_TRANSLATABLE)
         if value is not None:
-            val_bool = self._value_from_string(value, gobject.TYPE_BOOLEAN)
+            val_bool = util.get_bool_from_string_with_default(value, True)
             util.gtk_widget_set_metadata(obj,
                                          util.METADATA_I18N_IS_TRANSLATABLE,
                                          name,
@@ -351,7 +439,7 @@ class WidgetTree(object):
 
         value = xml_node.getAttribute(tags.XML_TAG_CONTEXT)
         if value is not None:
-            val_bool = self._value_from_string(value, gobject.TYPE_BOOLEAN)
+            val_bool = util.get_bool_from_string_with_default(value, True)            
             util.gtk_widget_set_metadata(obj,
                                          util.METADATA_I18N_HAS_CONTEXT,
                                          name,
@@ -370,7 +458,7 @@ class WidgetTree(object):
         name, value = util.xml_read_raw_property(xml_node)
         for pspec in gtk.container_class_list_child_properties(parent):
             if pspec.name == name:
-                value = self._value_from_string(value, pspec.value_type)
+                value = self._value_from_string(value, pspec)
                 if value is not None:
                     parent.child_set_property(widget, name, value)
                     break
@@ -380,26 +468,31 @@ class WidgetTree(object):
 
         name = xml_node.getAttribute(tags.XML_TAG_NAME)
         after = xml_node.getAttribute(tags.XML_TAG_AFTER)
-        after = self._value_from_string(after, gobject.TYPE_BOOLEAN)
         handler = xml_node.getAttribute(tags.XML_TAG_HANDLER)
 
         self._signals.append([widget, name, handler, after])
         
-    def _value_from_string(self, value, value_type):
-        if value_type in [gobject.TYPE_INT, gobject.TYPE_UINT]:
+    def _value_from_string(self, value, pspec):
+        
+        if hasattr(pspec, 'flags_class'):
+            return flags_from_string(value, pspec)
+        elif hasattr(pspec, 'enum_class'):
+            return enum_from_string(value, pspec)
+
+        if pspec.value_type in [gobject.TYPE_INT, gobject.TYPE_UINT]:
             # can be GParamUnichar
             try:
                 return int(value)
             except ValueError:
                 return value
-        elif value_type in [gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE]:
+        elif pspec.value_type in [gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE]:
             return float(value)
-        elif value_type == gobject.TYPE_BOOLEAN:
+        elif pspec.value_type == gobject.TYPE_BOOLEAN:
             if value in ['True', 'TRUE', 'yes', '1']:
                 return True
             else:
                 return False
-        elif value_type == gobject.TYPE_STRING:
+        elif pspec.value_type == gobject.TYPE_STRING:
             return value
 
 #        else:
@@ -437,7 +530,10 @@ class WidgetTree(object):
     def get_actions(self):
         return self._actions
     actions = property(get_actions)
-    
+
+    def get_widget(self, widget_name):
+        return self._widgets.get(widget_name, None)
+        
 if __name__ == '__main__':
     import sys
     wt = WidgetTree(sys.argv[1])
