@@ -17,7 +17,9 @@ import gobject
 
 from gazpacho import propertyclass as pc
 from gazpacho.loader import tags
+from gazpacho.choice import enum_to_string, flags_to_string
 from gazpacho.l10n import _
+from gazpacho import util
 
 from xml.sax.saxutils import escape
 
@@ -27,22 +29,22 @@ class Property(object):
     in the GladeProject. """
     
     def __init__(self, property_class, widget):
-        self._klass = property_class
-        self._widget = widget
-        self._enabled = True
-        self._loading = False
+        self.klass = property_class
+        self.widget = widget
+        self.enabled = True
+        self.loading = False
         self._value = None
-        self._is_translatable = True
-        self._i18n_comment = None
-        self._has_i18n_context = False   
+        self.is_translatable = True
+        self.i18n_comment = None
+        self.has_i18n_context = False   
 
         if property_class.type in (gobject.TYPE_DOUBLE, gobject.TYPE_INT,
                                    gobject.TYPE_FLOAT):
-            self._enabled = property_class.optional_default
+            self.enabled = property_class.optional_default
 
         initial_value = ''
         try:
-            initial_value = widget.get_property(property_class.id)
+            initial_value = widget.gtk_widget.get_property(property_class.id)
         except TypeError:
             pass
 
@@ -57,96 +59,57 @@ class Property(object):
 
 #        print 'Creating property %s of type %s with value' % (self._klass.id, pc.type2str(self._klass.type)), self._value, initial_value
 
-    # properties
-    def get_klass(self): return self._klass
-    klass = property(get_klass)
-
-    def get_widget(self): return self._widget
-    widget = property(get_widget)
-
-    def get_enabled(self): return self._enabled
-    def set_enabled(self, value): self._enabled = value
-    enabled = property(get_enabled, set_enabled)
-
-    def get_loading(self): return self._loading
-    def set_loading(self, value): self._loading = value
-    loading = property(get_loading, set_loading)
-
     def get_value(self):
         # Use custom get function if available, fallback to the
         # regular value.
         if self.klass._get_function is not None:
-            value = self._klass._get_function(self._widget.widget)
+            value = self.klass._get_function(self.widget.gtk_widget)
             if value is not None:
                 return value
 
         return self._value
 
     def set_value(self, value):
-        if self._klass.type in (gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE,
-                                gobject.TYPE_INT) \
-                                and value != self._klass.default:
-            self._enabled = True
+        if self.klass.type in (gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE,
+                               gobject.TYPE_INT) \
+           and value != self.klass.default:
+            self.enabled = True
                 
-        if self._klass.type == gobject.TYPE_BOOLEAN:
+        if self.klass.type == gobject.TYPE_BOOLEAN:
             self._value = bool(value)
-        elif self._klass.type in (gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE):
+        elif self.klass.type in (gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE):
             self._value = float(value or 0.0)
-        elif self._klass.type == gobject.TYPE_INT:
+        elif self.klass.type == gobject.TYPE_INT:
             self._value = int(value or 0)
-        elif self._klass.type == gobject.TYPE_STRING:
+        elif self.klass.type == gobject.TYPE_STRING:
             self._value = value or ""
-        elif self._klass.type == gobject.TYPE_UINT:
-            if self._klass.id in pc.UNICHAR_PROPERTIES:
+        elif self.klass.type == gobject.TYPE_UINT:
+            if self.klass.id in pc.UNICHAR_PROPERTIES:
                 self._value = unicode(value and value[0] or "")
             else:
                 self._value = int(value or 0)
-        elif self._klass.type == gobject.TYPE_ENUM:
+        elif self.klass.type == gobject.TYPE_ENUM:
             self._value = int(value or 0)
-        elif self._klass.type == gobject.TYPE_FLAGS:
+        elif self.klass.type == gobject.TYPE_FLAGS:
             self._value = int(value or 0)
-        elif self._klass.type == gobject.TYPE_OBJECT:
+        elif self.klass.type == gobject.TYPE_OBJECT:
             self._value = value
-        elif self._klass.type is None:
+        elif self.klass.type is None:
             print _("Error: the property %s should not be of type None") % \
-                  self._klass.name
+                  self.klass.name
             return
 
     value = property(get_value, set_value)
 
-    def get_is_translatable(self):
-        return self._is_translatable
-
-    def set_is_translatable(self, value):
-        self._is_translatable = value
-
-    is_translatable = property(get_is_translatable, set_is_translatable)
-
-    def get_has_i18n_context(self):
-        return self._has_i18n_context
-
-    def set_has_i18n_context(self, value):
-        self._has_i18n_context = value
-
-    has_i18n_context = property(get_has_i18n_context, set_has_i18n_context)
-
-    def get_i18n_comment(self):
-        return self._i18n_comment
-
-    def set_i18n_comment(self, value):
-        self._i18n_comment = value
-
-    i18n_comment = property(get_i18n_comment, set_i18n_comment)
-
     def _set_property(self, value):
-        if self._klass.packing:
-            parent = self._widget.get_parent()
-            container = parent.widget
-            child = self._widget.widget
-            container.child_set_property(child, self._klass.id, value)
+        if self.klass.packing:
+            parent = self.widget.get_parent()
+            container = parent.gtk_widget
+            child = self.widget.gtk_widget
+            container.child_set_property(child, self.klass.id, value)
         else:
             try:
-                self._widget.widget.set_property(self._klass.id, value)
+                self.widget.gtk_widget.set_property(self.klass.id, value)
             except TypeError:
                 pass # XXX TODO there is a bug in pygtk:
             # >>> e.set_property('invisible-char', unicode('*'))
@@ -155,21 +118,21 @@ class Property(object):
             # TypeError: could not convert argument to correct param type
 
     def set(self, value):
-        if not self._enabled: return
-        if self._loading: return
+        if not self.enabled: return
+        if self.loading: return
 
-        self._loading = True
+        self.loading = True
 
         # first, trigger the conversions
         self.value = value
         
         # if there is a custom set property use it
-        if self._klass._set_function:
-            self._klass._set_function(self._widget.widget, self._value)
+        if self.klass._set_function:
+            self.klass._set_function(self.widget.gtk_widget, self._value)
         else:
             self._set_property(self.value)
             
-        self._loading = False
+        self.loading = False
 
 
     def _internal_write(self, document, value):
@@ -199,8 +162,8 @@ class Property(object):
                 if self.i18n_comment is not None:
                     node.setAttribute(tags.XML_TAG_COMMENT, self.i18n_comment)
 
-#        if self.klass.default == self._value:
-#            return None
+        if self.klass.original_default == self._value:
+            return None
 
         text = document.createTextNode(value)
         node.appendChild(text)
@@ -209,9 +172,31 @@ class Property(object):
         
     def write(self, document):
         if self._value is not None:
-            return self._internal_write(document, escape(str(self._value)))
+            if self.klass.type == gobject.TYPE_ENUM:
+                value = enum_to_string(self._value, self.klass.spec)
+            elif self.klass.type == gobject.TYPE_FLAGS:
+                value = flags_to_string(self._value, self.klass.spec)
+            else:
+                value = escape(str(self._value))
+            return self._internal_write(document, value)
     
     def write_in_cdata(self, document):
         if self._value is not None:
             data = '<![CDATA[%s]]>' % self._value
             return self._internal_write(document, data)
+
+
+    def read_i18n_data(self, gtk_widget):
+        comment = util.gtk_widget_get_metadata(gtk_widget,
+                                               util.METADATA_I18N_COMMENT,
+                                               self.klass.id)
+        if comment is not None:
+            self.i18n_comment = comment
+
+        self.is_translatable = util.gtk_widget_get_metadata(gtk_widget,
+                                                            util.METADATA_I18N_IS_TRANSLATABLE,
+                                                            self.klass.id)
+
+        self.has_i18n_context = util.gtk_widget_get_metadata(gtk_widget,
+                                                             util.METADATA_I18N_HAS_CONTEXT,
+                                                             self.klass.id)
