@@ -2,7 +2,7 @@ import gtk
 import gobject
 
 from gazpacho.l10n import _
-from gazpacho.widget import Widget
+from gazpacho.widget import get_widget_from_gtk_widget
 from gazpacho.util import unselect_when_clicked_on_empty_space, select_iter
 from gazpacho.util import unselect_when_press_escape, xml_filter_nodes
 
@@ -37,7 +37,7 @@ class UIEditor(gtk.Dialog):
         
     def set_widget(self, widget, proxy):
         self.widget = widget
-        self.gwidget = Widget.get_from_gtk_widget(widget)
+        self.gwidget = get_widget_from_gtk_widget(widget)
         if isinstance(widget, gtk.MenuBar):
             self.set_title(_('Editing menubar %s') % self.gwidget.name)
             self.is_tree = True
