@@ -13,7 +13,7 @@ class GtkWindowTest(unittest.TestCase):
         app.palette.select_widget('Gtk+ Base', 'GtkWindow')
         refresh_gui()
         gwidget = project.get_widget_by_name('window1')
-        self.assertEqual(gtk.Window, type(gwidget.widget))
+        self.assertEqual(gtk.Window, type(gwidget.gtk_widget))
 
     def test02Property(self):
         global app, project
@@ -22,7 +22,7 @@ class GtkWindowTest(unittest.TestCase):
         cm = app.get_command_manager()
         cm.set_property(prop, 'Window Title Test')
         refresh_gui()
-        self.assertEqual(gwidget.widget.get_title(), 'Window Title Test')
+        self.assertEqual(gwidget.gtk_widget.get_title(), 'Window Title Test')
 
     def test03Saving(self):
         global project
